
#ifndef __GLCD_H
#define __GLCD_H

/* Includes ------------------------------------------------------------------*/
#include <stdlib.h>
#include <LPC17xx.h>
#include "lpc17xx_ssp.h"
#include "lpc17xx_gpio.h"
#include "lpc17xx_pinsel.h"

/* Private define ------------------------------------------------------------*/
#define MAX_X  240
#define MAX_Y  320

#define SPI_START   (0x70)              /* Start byte for SPI transfer        */
#define SPI_RD      (0x01)              /* WR bit 1 within start              */
#define SPI_WR      (0x00)              /* WR bit 0 within start              */
#define SPI_DATA    (0x02)              /* RS bit 1 within start byte         */
#define SPI_INDEX   (0x00)              /* RS bit 0 within start byte         */


/* PORT number that /CS pin assigned on */
#define CSB_PORT_NUM	0
/* PIN number that  /CS pin assigned on */
#define CSB_PIN_NUM		16

#define SPI_CS_LOW    GPIO_ClearValue(CSB_PORT_NUM, ( (uint32_t)1 << CSB_PIN_NUM ) )
#define SPI_CS_HIGH   GPIO_SetValue(CSB_PORT_NUM, ( (uint32_t) 1 << CSB_PIN_NUM ) )

#if use_ssp02
#define SPI_CS_LOW    {LPC_GPIO1->FIOCLR|=(1UL<<21);GPIO_ClearValue(0,((uint32_t)1<<16));}
#define SPI_CS_HIGH   {GPIO_SetValue(1,((uint32_t)1<< 21));GPIO_SetValue(0,((uint32_t)1<<16));}
#endif



/* LCD color */
#define White          0xFFFF
#define WHITE          0xFFFF
#define Black          0x0000
#define BLACK          0x0000
#define Grey           0xF7DE
#define Blue           0x001F
#define Blue2          0x051F
#define Red            0xF800
#define Magenta        0xF81F
#define Green          0x07E0
#define Cyan           0x7FFF
#define Yellow         0xFFE0
#define TRANSPARENT 1
/* Private function prototypes -----------------------------------------------*/
//===============================================
static void LCD_Configuration(void)
{
 	PINSEL_CFG_Type PinCfg;
    SSP_CFG_Type SSP_ConfigStruct;
	/*
	 * Initialize SPI pin connect
	 * P0.15 - LCD_CSB - used as GPIO
	 * P0.16 - LCD_SCK
	 * P0.17 - LCD_MISO
	 * P0.18 - LCD_MOSI
	 */
	PinCfg.Funcnum = 2;
	PinCfg.OpenDrain = 0;
	PinCfg.Pinmode = 0;
	PinCfg.Portnum = 0;
	PinCfg.Pinnum = 15;
	PINSEL_ConfigPin(&PinCfg);
	PinCfg.Pinnum = 17;
	PINSEL_ConfigPin(&PinCfg);
	PinCfg.Pinnum = 18;
	PINSEL_ConfigPin(&PinCfg);


#if use_ssp02
	//ssp0/2
	PinCfg.Funcnum = 3;
	PinCfg.Portnum = 1;
	PinCfg.Pinnum = 20;
	PINSEL_ConfigPin(&PinCfg);
	PinCfg.Pinnum = 23;
	PINSEL_ConfigPin(&PinCfg);
	PinCfg.Pinnum = 24;
	PINSEL_ConfigPin(&PinCfg);
#endif

	PinCfg.Funcnum = 0;
	PinCfg.Portnum = 0;
	PinCfg.Pinnum = 16;
	PINSEL_ConfigPin(&PinCfg);


#if use_ssp02
	//ssp0/2
	PinCfg.Portnum = 1;
	PinCfg.Pinnum = 21;
	PINSEL_ConfigPin(&PinCfg);
	PinCfg.Pinnum = 25;
	PINSEL_ConfigPin(&PinCfg);
#endif

    /* P1.31 LCD_CSB is output */
	   GPIO_SetDir(CSB_PORT_NUM, ( (uint32_t) 1 << CSB_PIN_NUM ), 1);
		GPIO_SetValue(CSB_PORT_NUM, ( (uint32_t) 1 << CSB_PIN_NUM ) );

//ssp0/2
#if use_ssp02
		GPIO_SetDir(1, ( (uint32_t) 1 << 21 ), 1);
		GPIO_SetValue(1, ( (uint32_t) 1 << 21 ) );
		GPIO_SetDir(1, ( (uint32_t) 1 << 25 ), 1);
		GPIO_ClearValue(1, ( (uint32_t) 1 << 25 ) );
delay_ms(50);
		GPIO_SetValue(1, ( (uint32_t) 1 << 25 ) );
#endif


	/* initialize SSP configuration structure to default */
	SSP_ConfigStructInit(&SSP_ConfigStruct);

	SSP_ConfigStruct.CPHA = SSP_CPHA_SECOND;
	SSP_ConfigStruct.CPOL = SSP_CPOL_LO;
	SSP_ConfigStruct.ClockRate = 25000000;

	/* Initialize SSP peripheral with parameter given in structure above */
	SSP_Init(LPC_SSP0, &SSP_ConfigStruct);
	/* Enable SSP peripheral */
	SSP_Cmd(LPC_SSP0, ENABLE);
}

/*******************************************************************************
* Function Name  : LPC17xx_SPI_SendRecvByte
* Description    : Send one byte then recv one byte of response
* Input          : - byte_s: byte_s
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
uint8_t LPC17xx_SPI_SendRecvByte (uint8_t byte_s)
{
	/* wait for current SSP activity complete */
	while (SSP_GetStatus(LPC_SSP0, SSP_STAT_BUSY) ==  SET);

	SSP_SendData(LPC_SSP0, (uint16_t) byte_s);

	while (SSP_GetStatus(LPC_SSP0, SSP_STAT_RXFIFO_NOTEMPTY) == RESET);
	return (SSP_ReceiveData(LPC_SSP0));
}

/*******************************************************************************
* Function Name  : LCD_WriteReg
* Description    : LCD�������Ĵ�����ַ
* Input          : - index: �Ĵ�����ַ
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
__inline void LCD_WriteIndex(uint8_t index)
{
	SPI_CS_LOW;
	LPC17xx_SPI_SendRecvByte(SPI_START | SPI_WR | SPI_INDEX);   /* Write : RS = 0, RW = 0       */
	LPC17xx_SPI_SendRecvByte(0);
	LPC17xx_SPI_SendRecvByte(index);
	SPI_CS_HIGH;
}

/*******************************************************************************
* Function Name  : LCD_WriteReg
* Description    : LCD�Ĵ������
* Input          : - index: �Ĵ������
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
__inline void LCD_WriteData(uint16_t data)
{
	SPI_CS_LOW;
	LPC17xx_SPI_SendRecvByte(SPI_START | SPI_WR | SPI_DATA);    /* Write : RS = 1, RW = 0       */
	LPC17xx_SPI_SendRecvByte((data >>   8));                    /* Write D8..D15                */
	LPC17xx_SPI_SendRecvByte((data & 0xFF));                    /* Write D0..D7                 */
	SPI_CS_HIGH;
}

/*******************************************************************************
* Function Name  : LCD_Write_Data_Start
* Description    : Start of data writing to the LCD controller
* Input          : None
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
__inline void LCD_Write_Data_Start(void)
{
  LPC17xx_SPI_SendRecvByte(SPI_START | SPI_WR | SPI_DATA);    /* Write : RS = 1, RW = 0       */
}

/*******************************************************************************
* Function Name  : LCD_Write_Data_Only
* Description    : Data writing to the LCD controller
* Input          : - data: data to be written
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
__inline void LCD_Write_Data_Only(uint16_t data)
{
  LPC17xx_SPI_SendRecvByte((data >>   8));      /* Write D8..D15                */
  LPC17xx_SPI_SendRecvByte((data & 0xFF));      /* Write D0..D7                 */
}

/*******************************************************************************
* Function Name  : LCD_ReadData
* Description    : ��ȡ���������
* Input          : None
* Output         : None
* Return         : ���ض�ȡ�������
* Attention		 : None
*******************************************************************************/
uint16_t LCD_ReadData(void)
{
	uint16_t value;

	SPI_CS_LOW;
	LPC17xx_SPI_SendRecvByte(SPI_START | SPI_RD | SPI_DATA);    /* Read: RS = 1, RW = 1         */
	LPC17xx_SPI_SendRecvByte(0);                                /* Dummy read 1                 */
	value   = LPC17xx_SPI_SendRecvByte(0);                      /* Read D8..D15                 */
	value <<= 8;
	value  |= LPC17xx_SPI_SendRecvByte(0);                      /* Read D0..D7                  */
	SPI_CS_HIGH;

	return value;
}

/*******************************************************************************
* Function Name  : LCD_WriteReg
* Description    : Writes to the selected LCD register.
* Input          : - LCD_Reg: address of the selected register.
*                  - LCD_RegValue: value to write to the selected register.
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
__inline void LCD_WriteReg(uint16_t LCD_Reg,uint16_t LCD_RegValue)
{
	/* Write 16-bit Index, then Write Reg */
	LCD_WriteIndex(LCD_Reg);
	/* Write 16-bit Reg */
	LCD_WriteData(LCD_RegValue);
}

/*******************************************************************************
* Function Name  : LCD_WriteReg
* Description    : Reads the selected LCD Register.
* Input          : None
* Output         : None
* Return         : LCD Register Value.
* Attention		 : None
*******************************************************************************/
 uint16_t LCD_ReadReg(uint16_t LCD_Reg)
{
	uint16_t LCD_RAM;

	/* Write 16-bit Index (then Read Reg) */
	LCD_WriteIndex(LCD_Reg);
	/* Read 16-bit Reg */
	LCD_RAM = LCD_ReadData();
	return LCD_RAM;
}

/*******************************************************************************
* Function Name  : LCD_SetCursor
* Description    : Sets the cursor position.
* Input          : - Xpos: specifies the X position.
*                  - Ypos: specifies the Y position.
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
static void LCD_SetCursorxxx( uint16_t Xpos, uint16_t Ypos )
{
	/* 0x9320 */
	LCD_WriteReg(0x0020, Xpos );
	LCD_WriteReg(0x0021, Ypos );
}

/*******************************************************************************
* Function Name  : LCD_Delay
* Description    : Delay Time
* Input          : - nCount: Delay Time
* Output         : None
* Return         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
 static void delay_ms(uint16_t ms)
{
	uint16_t i,j;
	for( i = 0; i < ms; i++ )
	{
		for( j = 0; j < 1141; j++ );
	}
}

/*******************************************************************************
* Function Name  : LCD_Initializtion
* Description    : Initialize TFT Controller.
* Input          : None
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
void LCD_Initializtion(void)
{
	uint16_t DeviceCode;

	LCD_Configuration();
	DeviceCode = LCD_ReadReg(0x0000);		/* Read ID	*/
	/* Different driver IC initialization*/
	if( DeviceCode == 0x9320 || DeviceCode == 0x9300 )
	{
		LCD_WriteReg(0x00,0x0000);
		LCD_WriteReg(0x01,0x0100); /* Driver Output Contral */
		LCD_WriteReg(0x02,0x0700); /* LCD Driver Waveform Contral */
		LCD_WriteReg(0x03,0x1038); /* Set the scan mode */
		LCD_WriteReg(0x04,0x0000); /* Scalling Contral */
		LCD_WriteReg(0x08,0x0202); /* Display Contral 2 */
		LCD_WriteReg(0x09,0x0000); /* Display Contral 3 */
		LCD_WriteReg(0x0a,0x0000); /* Frame Cycle Contal */
		LCD_WriteReg(0x0c,(1<<0)); /* Extern Display Interface Contral 1 */
		LCD_WriteReg(0x0d,0x0000); /* Frame Maker Position */
		LCD_WriteReg(0x0f,0x0000); /* Extern Display Interface Contral 2 */
		delay_ms(50);
		LCD_WriteReg(0x07,0x0101); /* Display Contral */
		delay_ms(50);
		LCD_WriteReg(0x10,(1<<12)|(0<<8)|(1<<7)|(1<<6)|(0<<4)); /* Power Control 1 */
		LCD_WriteReg(0x11,0x0007);                              /* Power Control 2 */
		LCD_WriteReg(0x12,(1<<8)|(1<<4)|(0<<0));                /* Power Control 3 */
		LCD_WriteReg(0x13,0x0b00);                              /* Power Control 4 */
		LCD_WriteReg(0x29,0x0000);                              /* Power Control 7 */
		LCD_WriteReg(0x2b,(1<<14)|(1<<4));

		LCD_WriteReg(0x50,0);       /* Set X Start */
		LCD_WriteReg(0x51,239);	    /* Set X End */
		LCD_WriteReg(0x52,0);	    /* Set Y Start */
		LCD_WriteReg(0x53,319);	    /* Set Y End */
		delay_ms(50);

		LCD_WriteReg(0x60,0x2700); /* Driver Output Control */
		LCD_WriteReg(0x61,0x0001); /* Driver Output Control */
		LCD_WriteReg(0x6a,0x0000); /* Vertical Srcoll Control */

		LCD_WriteReg(0x80,0x0000); /* Display Position? Partial Display 1 */
		LCD_WriteReg(0x81,0x0000); /* RAM Address Start? Partial Display 1 */
		LCD_WriteReg(0x82,0x0000); /* RAM Address End-Partial Display 1 */
		LCD_WriteReg(0x83,0x0000); /* Displsy Position? Partial Display 2 */
		LCD_WriteReg(0x84,0x0000); /* RAM Address Start? Partial Display 2 */
		LCD_WriteReg(0x85,0x0000); /* RAM Address End? Partial Display 2 */

		LCD_WriteReg(0x90,(0<<7)|(16<<0)); /* Frame Cycle Contral */
		LCD_WriteReg(0x92,0x0000);         /* Panel Interface Contral 2 */
		LCD_WriteReg(0x93,0x0001);         /* Panel Interface Contral 3 */
		LCD_WriteReg(0x95,0x0110);         /* Frame Cycle Contral */
		LCD_WriteReg(0x97,(0<<8));
		LCD_WriteReg(0x98,0x0000);         /* Frame Cycle Contral */
		LCD_WriteReg(0x07,0x0133);
	}
    delay_ms(100);   /* delay 50 ms */
}

/*******************************************************************************
* Function Name  : LCD_Clear
* Description    : Fill the screen as the specified color
* Input          : - Color: Screen Color
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
void LCD_Clear(uint16_t Color)
{
	uint32_t index=0;

	LCD_SetCursor(0,0);

	LCD_WriteIndex(0x0022);

 	SPI_CS_LOW;
	LCD_Write_Data_Start();

	for( index = 0; index < MAX_X * MAX_Y; index++ )
	{
		LCD_Write_Data_Only(Color);
	}
	SPI_CS_HIGH;
}

/******************************************************************************
* Function Name  : LCD_BGR2RGB
* Description    : RRRRRGGGGGGBBBBB To BBBBBGGGGGGRRRRR
* Input          : - color: BRG Color value
* Output         : None
* Return         : RGB Color value
* Attention		 : None
*******************************************************************************/
static uint16_t LCD_BGR2RGB(uint16_t color)
{
	uint16_t  r, g, b, rgb;

	b = ( color>>0 )  & 0x1f;
	g = ( color>>5 )  & 0x3f;
	r = ( color>>11 ) & 0x1f;

	rgb =  (b<<11) + (g<<5) + (r<<0);

	return( rgb );
}

/******************************************************************************
* Function Name  : LCD_GetPoint
* Description    : Get color value for the specified coordinates
* Input          : - Xpos: Row Coordinate
*                  - Xpos: Line Coordinate
* Output         : None
* Return         : Screen Color
* Attention		 : None
*******************************************************************************/
uint16_t LCD_GetPoint(uint16_t Xpos,uint16_t Ypos)
{
	uint16_t dummy;

	LCD_SetCursor(Xpos,Ypos);

	LCD_WriteIndex(0x0022);

	dummy = LCD_ReadData();   /* An empty read */
	dummy = LCD_ReadData();

	return  LCD_BGR2RGB( dummy );
}

/******************************************************************************
* Function Name  : LCD_SetPoint
* Description    : Drawn at a specified point coordinates
* Input          : - Xpos: Row Coordinate
*                  - Ypos: Line Coordinate
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
void LCD_SetPoint(uint16_t Xpos,uint16_t Ypos,uint16_t point)
{
	if( Xpos >= MAX_X || Ypos >= MAX_Y )
	{
		return;
	}
	LCD_SetCursor(Xpos,Ypos);
	LCD_WriteReg(0x0022,point);
}

/******************************************************************************
* Function Name  : LCD_DrawLine
* Description    : Bresenham's line algorithm
* Input          : - x1: A�������
*                  - y1: A�������
*				   - x2: B�������
*				   - y2: B�������
*				   - color: ����ɫ
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/

/*********************************************************************************************************
      END FILE
*********************************************************************************************************/
#include "lib_fonts.h"
#define Gui_DrawPoint LCD_SetPoint
volatile uint8_t xbuffer[16];

uint8_t *ptr_image=0, lcd_zeilen_versatz, ascii_versatz=32;
uint8_t *ptr_font=f8x12,f_width=8, f_height=12,vertikal_zeichnen=0, frame=0;
uint8_t frame_horz_versatz,frame_vert_versatz;
uint16_t curposx, curposy, Xpos,Ypos;
uint16_t backcolor=WHITE,frontcolor=BLACK;


 void LCD_SetCursor( uint16_t Xpos, uint16_t Ypos )
{
	/* 0x9320 */
	LCD_WriteReg(0x0020, MAX_X -Ypos );
	LCD_WriteReg(0x0021, Xpos );
}

void lcd_goto(char Zeile,char Spalte){
curposx=(Spalte)*(f_width+frame);
curposy=(Zeile-1)*(f_height+2*frame)+lcd_zeilen_versatz;
//if(frame)curposy+=2;//-
}

void selectfont(char abfr){
switch (abfr){
case (1):
ptr_font=f5x7;f_width=5; f_height=7;vertikal_zeichnen=1; frame=1;lcd_zeilen_versatz=9;
ascii_versatz=32,frame_vert_versatz=0,frame_horz_versatz=0;
break;
case (2):
ptr_font=f8x12;f_width=8; f_height=12;vertikal_zeichnen=0; frame=0;lcd_zeilen_versatz=0;
ascii_versatz=32,frame_vert_versatz=0,frame_horz_versatz=0;
break;
case (3):
ptr_font=f3x5;f_width=3; f_height=5;vertikal_zeichnen=0; frame=1;lcd_zeilen_versatz=0;
ascii_versatz=32,frame_vert_versatz=0,frame_horz_versatz=4;
break;
}
}
//=============================================
void PutChar( uint8_t ASCI)
 {	uint16_t i, j;
     uint16_t  tmp_char;
 //Fontheader auswerten
 //f_width= pgm_read_byte( &ptr_font[2]);
 //f_height=pgm_read_byte( &ptr_font[3]);
 //frame =pgm_read_byte( &ptr_font[4]);
 //vertikal_zeichnen=pgm_read_byte( &ptr_font[5]);
 //if (frame==1){f_width++;f_height+=2;}


 u16 Xpos=curposx;
 u16 Ypos=curposy;
 if(frame){
 //	if(!vertikal_zeichnen)Xpos=curposx-1+f_height;

 for(i=0;i<f_width +1;i++){
 //Gui_DrawPoint(Xpos+f_height+1-frame_vert_versatz,Ypos+f_width-i-1, backcolor );}
 Gui_DrawPoint(Xpos-frame_vert_versatz+i,Ypos+1+frame_horz_versatz, backcolor );}

 for(i=0;i<f_width +1;i++){
 Gui_DrawPoint( Xpos-frame_vert_versatz+i , Ypos-f_height+frame_horz_versatz,      backcolor );}

 for(i=0;i<f_height+1;i++){
 Gui_DrawPoint( Xpos-frame_vert_versatz,Ypos-i+frame_horz_versatz, backcolor );}

 //pos 1nach unten:
 Xpos++;//90°Bildschirm sonst ypos++
 }
 Xpos=curposx+1;




 if(vertikal_zeichnen){
 //Gui_DrawPoint(20,20,BLACK);
         for( j=0; j<f_width; j++ ){

  //      tmp_char = buffer[i];
  //      	 tmp_char = pgm_read_byte( &ptr_font[(ASCI-32)*f_width+j+7]);
        	 tmp_char = ( ptr_font[(ASCI-32)*f_width+j+7]);
 //lcd_int3(tmp_char);lw(" ");
     for( i=0; i<f_height; i++ ){
             if( (tmp_char >>(f_height-1) - i) & 0x01 == 0x01 ){
                 Gui_DrawPoint( Xpos +j , Ypos -i , frontcolor );  // Character color
                // Gui_DrawPoint( Xpos +i , Ypos +j , frontcolor );  // Character color
             }else{
 			if(backcolor!=TRANSPARENT)
 			Gui_DrawPoint( Xpos +j, Ypos -i , backcolor );  // Background color
 			//Gui_DrawPoint( Xpos + i, Ypos +j , backcolor );  // Background color

             }
         }
     }
 }else{
 //Xpos=curposx+2-f_height;
 /**/
 for( i=0; i<f_height; i++ ){




 //       tmp_char = buffer[i];
	// tmp_char = pgm_read_byte( &ptr_font[(ASCI-ascii_versatz)*f_height+i+7]);
	 tmp_char = ( ptr_font[(ASCI-ascii_versatz)*f_height+i+7]);
 //tmp_char = ( f8x12[(ASCI-ascii_versatz)*f_height+i+7]);
 //testx[i]=f8x12[7+i+(ASCI-ascii_versatz)*f_height];
 //testy[i]=7+i+(ASCI-ascii_versatz)*f_height;
 //lcd_int3(tmp_char);lw(" ");

 for( j=0; j<f_width; j++ ){
             if( (tmp_char >>(f_width-1) - j) & 0x01 == 0x01 ){
      Gui_DrawPoint( Xpos +j , Ypos +i , frontcolor );  // Character color
  //  Gui_DrawPoint( Xpos -i , Ypos +j , frontcolor );  // Character color
             }else{
         if(backcolor!=TRANSPARENT)
 		Gui_DrawPoint( Xpos + j, Ypos +i , backcolor );
 //      Gui_DrawPoint( Xpos - i, Ypos +j , backcolor );  // Background color
             }
         }
     }
 	}
 }

//checked
void GUI_Text(uint8_t *str)
{
	uint8_t TempChar;
    do
    {
        TempChar = *str++;
        PutChar(TempChar);

           curposx =curposx+ f_width;
		   if(frame)curposx++;
 //relposx=relposx+ f_width;
    }
    while ( *str != 0 );
}
#define lcd_write GUI_Text



#define glcd_goto lcd_goto
#define glcd_write lcd_write
#define LcdClear LCD_Clear
#define LcdClearZeilen(x,y,z) LCD_Clear(z)
#define glcd_pix Gui_DrawPoint
#define sub_set_Comma() {;}
#define glcd_3x5(x,y,z) {curposx=x;curposy=y;lcd_int(z);}
#define lgi(x,y,z) {lcd_goto(x,y);lcd_int(z);}
#define lwi(x,y,z) {lcd_goto(x,y);lcd_write(z);}
//#define glcd_3x5(x,y,z) {;}


























































#endif

/*********************************************************************************************************
      END FILE
*********************************************************************************************************/
